@@ -41,7 +41,7 @@ fn codegen(fix_dictionary: Dictionary, filename: &str) -> io::Result<()> {
     // writing.
     let dir = PathBuf::from(var("OUT_DIR").unwrap());
     let codegen_settings = &mut codegen::Settings::default();
-    codegen_settings.set_fefix_crate_name("crate");
+    codegen_settings.fefix_crate_name = "crate".to_string();
     let code = codegen::gen_definitions(fix_dictionary, &codegen_settings);
     let path = dir.join(filename);
     let file = &mut File::create(path)?;
