@@ -36,70 +36,53 @@
 //! |**Decoder type**      |Operates on              |Produces    |
 //! |----------------------|-------------------------|------------|
 //! |[`RawDecoder`]        |`&[u8]`                  |[`RawFrame`]|
+//! |[`RawDecoderBuffered`]|byte streams             |[`RawFrame`]|
 //! |[`Decoder`]           |`&[u8]`                  |[`Message`] |
-//! |[`RawDecoderBuffered`]|`Vec<u8>` internal buffer|[`RawFrame`]|
-//! |[`DecoderBuffered`]   |`Vec<u8>` internal buffer|[`Message`] |
+//! |[`DecoderBuffered`]   |data streams             |[`Message`] |
 
 use crate::dict::IsFieldDefinition;
 use crate::FixValue;
-use std::fmt;
 use std::fmt::Debug;
-use std::io;
 
 mod config;
 mod decoder;
 mod encoder;
 mod field_access;
+mod field_locator;
 mod raw_decoder;
-#[cfg(feature = "utils-tokio")]
-mod tokio_decoder;
 mod utils;
 
 pub use config::{Config, Configure};
-pub use decoder::{Decoder, DecoderBuffered, Fields, Message, MessageGroup, MessageGroupEntry};
+pub use decoder::{Decoder, DecoderBuffered, Fields, Message, MessageGroup};
 pub use encoder::{Encoder, EncoderHandle};
 pub use field_access::{FieldAccess, RepeatingGroup};
+pub use field_locator::{FieldLocator, FieldLocatorContext};
 pub use raw_decoder::{RawDecoder, RawDecoderBuffered, RawFrame};
+
+#[cfg(feature = "utils-tokio")]
+mod tokio_decoder;
 #[cfg(feature = "utils-tokio")]
 pub use tokio_decoder::TokioDecoder;
 
 /// The type returned in the event of an error during message decoding.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Debug, thiserror::Error)]
 pub enum DecodeError {
+    /// Mandatory field not found.
+    #[error("Field not found.")]
     FieldPresence,
-    /// Invalid FIX message syntax.
+    /// Invalid FIX message syntax, `BodyLength <9>` value mismatch, or similar errors.
+    #[error("Invalid FIX message syntax.")]
     Invalid,
-    Length,
+    /// Invalid `CheckSum <10>` FIX field value.
+    #[error("Invalid `CheckSum <10>` FIX field value.")]
     CheckSum,
+    /// I/O error.
+    #[error("I/O error: {0}")]
+    IO(#[from] std::io::Error),
 }
 
-impl fmt::Display for DecodeError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "SuperError is here!")
-    }
-}
-
-impl std::error::Error for DecodeError {
-    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
-        None
-    }
-}
-
-impl From<io::Error> for DecodeError {
-    fn from(_err: io::Error) -> Self {
-        Self::Invalid // FIXME
-    }
-}
-
-pub trait FvWrite<'a> {
-    type Key;
-
-    fn set_fv_with_key<'b, T>(&'b mut self, key: &Self::Key, value: T)
+pub trait FvWrite<'a, F> {
+    fn set_fv<'b, V>(&'b mut self, field: &F, value: V)
     where
-        T: FixValue<'b>;
-
-    fn set_fv<'b, V, F>(&'b mut self, field: &F, value: V)
-    where
-        V: FixValue<'b>,
-        F: IsFieldDefinition;
+        V: FixValue<'b>;
 }
