@@ -1,41 +1,26 @@
 use crate::tagvalue::{utils, Config, Configure, DecodeError};
+use crate::GetConfig;
 use std::ops::Range;
 
 /// An immutable view over the contents of a FIX message by a [`RawDecoder`].
 #[derive(Debug)]
-pub struct RawFrame<T>
-where
-    T: AsRef<[u8]>,
-{
-    data: T,
-    begin_string: Range<usize>,
-    payload: Range<usize>,
-    payload_offset: usize,
+pub struct RawFrame<T> {
+    pub data: T,
+    pub begin_string: Range<usize>,
+    pub payload: Range<usize>,
 }
 
 impl<T> RawFrame<T>
 where
     T: AsRef<[u8]>,
 {
-    pub fn map_data<F, M>(self, f: F) -> RawFrame<M>
-    where
-        F: Fn(T) -> M,
-        M: AsRef<[u8]>,
-    {
-        RawFrame {
-            data: f(self.data),
-            begin_string: self.begin_string,
-            payload: self.payload,
-            payload_offset: self.payload_offset,
-        }
-    }
-
     /// Returns an immutable reference to the raw contents of `self`.
     ///
     /// # Examples
     ///
     /// ```
     /// use fefix::tagvalue::{Config, RawDecoder};
+    /// use fefix::prelude::*;
     ///
     /// let mut decoder = RawDecoder::<Config>::new();
     /// decoder.config_mut().set_separator(b'|');
@@ -55,6 +40,7 @@ where
     ///
     /// ```
     /// use fefix::tagvalue::{Config, RawDecoder};
+    /// use fefix::prelude::*;
     ///
     /// let mut decoder = RawDecoder::<Config>::new();
     /// decoder.config_mut().set_separator(b'|');
@@ -80,6 +66,7 @@ where
     ///
     /// ```
     /// use fefix::tagvalue::{Config, RawDecoder};
+    /// use fefix::prelude::*;
     ///
     /// let mut decoder = RawDecoder::<Config>::new();
     /// decoder.config_mut().set_separator(b'|');
@@ -91,11 +78,6 @@ where
     pub fn payload(&self) -> &[u8] {
         &self.as_bytes()[self.payload.clone()]
     }
-
-    /// Returns the offset of [`RawFrame::payload`].
-    pub fn payload_offset(&self) -> usize {
-        self.payload_offset
-    }
 }
 
 /// A bare-bones FIX decoder for low-level message handling.
@@ -105,10 +87,7 @@ where
 /// `BodyLength (9)` and `CheckSum (10)` to the final user. Everything else is
 /// left to the user to deal with.
 #[derive(Debug, Clone, Default)]
-pub struct RawDecoder<C = Config>
-where
-    C: Configure,
-{
+pub struct RawDecoder<C = Config> {
     config: C,
 }
 
@@ -121,191 +100,206 @@ where
         Self::default()
     }
 
-    /// Creates a new [`RawDecoder`] with `config` as a [`Configure`]
-    /// implementor.
-    pub fn with_config(config: C) -> Self {
-        Self { config }
-    }
-
-    /// Turns `self` into a [`RawDecoderBuffered`] by adding an internal buffer.
+    /// Turns `self` into a [`RawDecoderBuffered`], allocating an internal
+    /// buffer.
     pub fn buffered(self) -> RawDecoderBuffered<C> {
         RawDecoderBuffered {
+            config: self.config,
             buffer: Vec::new(),
-            decoder: self,
-            error: None,
+            last_parser_state: ParserState::Empty,
         }
     }
 
-    /// Returns an immutable reference to the [`Configure`] implementor used by
-    /// `self`.
-    pub fn config(&self) -> &C {
-        &self.config
-    }
-
-    /// Returns a mutable reference to the [`Configure`] implementor used by
-    /// `self`.
-    pub fn config_mut(&mut self) -> &mut C {
-        &mut self.config
-    }
-
     /// Does minimal parsing on `data` and returns a [`RawFrame`] if it's valid.
     pub fn decode<T>(&self, src: T) -> Result<RawFrame<T>, DecodeError>
     where
         T: AsRef<[u8]>,
     {
         let data = src.as_ref();
-        if data.len() < utils::MIN_FIX_MESSAGE_LEN_IN_BYTES {
-            return Err(DecodeError::Length);
+        let len = data.len();
+        if len < utils::MIN_FIX_MESSAGE_LEN_IN_BYTES {
+            return Err(DecodeError::Invalid);
         }
-        let info = HeaderInfo::parse(data, self.config().separator())?;
-        utils::verify_body_length(data, info.start_of_body(), info.body_range().len())?;
-        if self.config().verify_checksum() {
+
+        let header_info =
+            HeaderInfo::parse(data, self.config().separator()).ok_or(DecodeError::Invalid)?;
+
+        utils::verify_body_length(
+            data,
+            header_info.field_1.end + 1,
+            header_info.nominal_body_len as usize,
+        )?;
+
+        if self.config.verify_checksum() {
             utils::verify_checksum(data)?;
         }
+
         Ok(RawFrame {
             data: src,
-            begin_string: info.begin_string_range(),
-            payload: info.body_range(),
-            payload_offset: info.body_range().start,
+            begin_string: header_info.field_0,
+            payload: header_info.field_1.end + 1..len - utils::FIELD_CHECKSUM_LEN_IN_BYTES,
         })
     }
 }
 
+impl<C> GetConfig for RawDecoder<C> {
+    type Config = C;
+
+    fn config(&self) -> &C {
+        &self.config
+    }
+
+    fn config_mut(&mut self) -> &mut C {
+        &mut self.config
+    }
+}
+
+#[derive(Debug)]
+enum ParserState {
+    Empty,
+    Header(HeaderInfo, usize),
+    Err(DecodeError),
+}
+
 /// A [`RawDecoder`] that can buffer incoming data and read a stream of messages.
-#[derive(Debug, Clone)]
-pub struct RawDecoderBuffered<C = Config>
-where
-    C: Configure,
-{
+#[derive(Debug)]
+pub struct RawDecoderBuffered<C = Config> {
+    config: C,
     buffer: Vec<u8>,
-    decoder: RawDecoder<C>,
-    error: Option<DecodeError>,
+    last_parser_state: ParserState,
 }
 
 impl<C> RawDecoderBuffered<C>
 where
     C: Configure,
 {
-    /// Returns an immutable reference to the [`Configure`] implementor used by
-    /// `self`.
-    pub fn config(&self) -> &C {
-        self.decoder.config()
-    }
-
-    /// Returns a mutable reference to the [`Configure`] implementor used by
-    /// `self`.
-    pub fn config_mut(&mut self) -> &mut C {
-        self.decoder.config_mut()
-    }
-
+    /// Empties all contents of the internal buffer of `self`.
     pub fn clear(&mut self) {
         self.buffer.clear();
-        self.error = None;
+        self.last_parser_state = ParserState::Empty;
     }
 
     /// Provides a buffer that must be filled before re-attempting to deserialize
     /// the next [`RawFrame`].
+    ///
+    /// # Panics
+    ///
+    /// Panics if the last call to [`RawDecoderBuffered::raw_frame`]
+    /// returned an [`Err`].
     pub fn supply_buffer(&mut self) -> &mut [u8] {
-        if self.buffer.as_slice().len() < utils::MIN_FIX_MESSAGE_LEN_IN_BYTES {
-            let current_len = self.buffer.as_slice().len();
-            let missing_len = utils::MIN_FIX_MESSAGE_LEN_IN_BYTES - current_len;
-            debug_assert!(missing_len > 0);
-            self.buffer.resize(missing_len, 0);
-            &mut self.buffer.as_mut_slice()[current_len..]
-        } else {
-            match HeaderInfo::parse(self.buffer.as_slice(), self.config().separator()) {
-                Ok(info) => {
-                    let start_of_body = info.start_of_body();
-                    let body_len = info.body_range().len();
-                    let total_len = start_of_body + body_len + utils::FIELD_CHECKSUM_LEN_IN_BYTES;
-                    let current_len = self.buffer.as_slice().len();
-                    self.buffer.resize(total_len, 0);
-                    &mut self.buffer.as_mut_slice()[current_len..]
-                }
-                Err(e) => {
-                    self.error = Some(e);
-                    &mut []
-                }
+        match self.last_parser_state {
+            ParserState::Empty => {
+                // There's no point in validating a FIX message that is too short to
+                // ever be valid.
+                self.buffer.resize(utils::MIN_FIX_MESSAGE_LEN_IN_BYTES, 0);
+                self.buffer.as_mut_slice()
+            }
+            ParserState::Header(_, expected_len) => {
+                let old_len = self.buffer.as_slice().len();
+                self.buffer.resize(expected_len, 0);
+                &mut self.buffer.as_mut_slice()[old_len..]
+            }
+            ParserState::Err(_) => {
+                panic!("This decoder is not valid anymore and it shouldn't have been used.")
             }
         }
     }
 
-    pub fn current_frame<'a>(&'a self) -> Result<Option<RawFrame<&'a [u8]>>, DecodeError> {
-        if let Some(err) = self.error.clone() {
-            Err(err)
-        } else {
-            let data = &self.buffer.as_slice();
-            if data.len() == 0 || data.len() == utils::MIN_FIX_MESSAGE_LEN_IN_BYTES {
-                Ok(None)
-            } else {
-                self.decoder.decode(*data).map(|message| Some(message))
+    pub fn parse(&mut self) {
+        match self.last_parser_state {
+            ParserState::Empty => {
+                let header_info =
+                    HeaderInfo::parse(self.buffer.as_slice(), self.config().separator());
+                if let Some(header_info) = header_info {
+                    let expected_len_of_frame = header_info.field_1.end
+                        + 1
+                        + header_info.nominal_body_len
+                        + utils::FIELD_CHECKSUM_LEN_IN_BYTES;
+
+                    self.last_parser_state =
+                        ParserState::Header(header_info, expected_len_of_frame);
+                } else {
+                    self.last_parser_state = ParserState::Err(DecodeError::Invalid);
+                }
+            }
+            ParserState::Header(_, _) => {}
+            ParserState::Err(_) => {}
+        }
+    }
+
+    pub fn raw_frame<'a>(&'a self) -> Result<Option<RawFrame<&'a [u8]>>, DecodeError> {
+        match &self.last_parser_state {
+            ParserState::Empty => Ok(None),
+            ParserState::Err(e) => match e {
+                DecodeError::CheckSum => Err(DecodeError::CheckSum),
+                DecodeError::Invalid => Err(DecodeError::Invalid),
+                DecodeError::FieldPresence => Err(DecodeError::FieldPresence),
+                DecodeError::IO(_) => unreachable!("Can't have an I/O error here."),
+            },
+            ParserState::Header(header_info, _len) => {
+                let data = &self.buffer.as_slice();
+
+                Ok(Some(RawFrame {
+                    data,
+                    begin_string: header_info.field_0.clone(),
+                    payload: header_info.field_1.end + 1
+                        ..data.len() - utils::FIELD_CHECKSUM_LEN_IN_BYTES,
+                }))
             }
         }
     }
 }
 
-// Information regarding the indices of "important" parts of the FIX message.
+impl<C> GetConfig for RawDecoderBuffered<C> {
+    type Config = C;
+
+    fn config(&self) -> &C {
+        &self.config
+    }
+
+    fn config_mut(&mut self) -> &mut C {
+        &mut self.config
+    }
+}
+
+#[derive(Debug, Clone)]
 struct HeaderInfo {
-    i_equal_sign: [usize; 2],
-    i_sep: [usize; 2],
-    body_length: usize,
+    field_0: Range<usize>,
+    field_1: Range<usize>,
+    nominal_body_len: usize,
 }
 
 impl HeaderInfo {
-    fn empty() -> Self {
-        Self {
-            i_equal_sign: [0, 0],
-            i_sep: [0, 0],
-            body_length: 0,
-        }
-    }
+    fn parse(data: &[u8], separator: u8) -> Option<Self> {
+        let mut info = Self {
+            field_0: 0..1,
+            field_1: 0..1,
+            nominal_body_len: 0,
+        };
 
-    pub fn start_of_body(&self) -> usize {
-        // The body starts at the character immediately after the separator of
-        // `BodyLength`.
-        self.i_sep[1] + 1
-    }
-
-    pub fn begin_string_range(&self) -> Range<usize> {
-        self.i_equal_sign[0] + 1..self.i_sep[0]
-    }
-
-    pub fn body_range(&self) -> Range<usize> {
-        let start = self.start_of_body();
-        start..start + self.body_length
-    }
-
-    fn parse(data: &[u8], separator: u8) -> Result<Self, DecodeError> {
-        let mut info = HeaderInfo::empty();
-        let mut field_i = 0;
+        let mut iterator = data.iter();
+        let mut find_byte = |byte| iterator.position(|b| *b == byte);
         let mut i = 0;
-        while field_i < 2 && i < data.len() {
-            let byte = data[i];
-            if byte == b'=' {
-                info.i_equal_sign[field_i] = i;
-                info.body_length = 0;
-            } else if byte == separator {
-                info.i_sep[field_i] = i;
-                field_i += 1;
-            } else {
-                info.body_length = info
-                    .body_length
-                    .wrapping_mul(10)
-                    .wrapping_add(byte.wrapping_sub(b'0') as usize);
-            }
-            i += 1;
-        }
-        // Let's check that we got valid values for everything we need.
-        if info.i_equal_sign[0] != 0
-            && info.i_equal_sign[1] != 0
-            && info.i_sep[0] != 0
-            && info.i_sep[1] != 0
-        {
-            debug_assert!(info.i_sep[1] < data.len());
-            Ok(info)
-        } else {
-            Err(DecodeError::Invalid)
+
+        i += find_byte(b'=')? + 1;
+        info.field_0.start = i;
+        i += find_byte(separator)?;
+        info.field_0.end = i;
+        i += 1;
+
+        i += find_byte(b'=')? + 1;
+        info.field_1.start = i;
+        i += find_byte(separator)?;
+        info.field_1.end = i;
+
+        for byte in &data[info.field_1.clone()] {
+            info.nominal_body_len = info
+                .nominal_body_len
+                .wrapping_mul(10)
+                .wrapping_add(byte.wrapping_sub(b'0') as usize);
         }
+
+        Some(info)
     }
 }
 
@@ -316,7 +310,10 @@ mod test {
     fn new_decoder() -> RawDecoder {
         let mut config = Config::default();
         config.set_separator(b'|');
-        RawDecoder::with_config(config)
+
+        let mut decoder = RawDecoder::new();
+        *decoder.config_mut() = config;
+        decoder
     }
 
     #[test]
@@ -324,7 +321,7 @@ mod test {
         let decoder = new_decoder();
         assert!(matches!(
             decoder.decode(&[] as &[u8]),
-            Err(DecodeError::Length)
+            Err(DecodeError::Invalid)
         ));
     }
 
@@ -350,12 +347,12 @@ mod test {
     fn message_with_empty_payload_is_invalid() {
         let decoder = new_decoder();
         let msg = "8=?|9=5|10=082|".as_bytes();
-        assert!(matches!(decoder.decode(msg), Err(DecodeError::Length)));
+        assert!(matches!(decoder.decode(msg), Err(DecodeError::Invalid)));
     }
 
     #[test]
     fn message_with_bad_checksum_is_invalid() {
-        let decoder = &mut new_decoder();
+        let mut decoder = new_decoder();
         decoder.config_mut().set_verify_checksum(true);
         let msg = "8=FIX.4.2|9=40|35=D|49=AFUNDMGR|56=ABROKER|15=USD|59=0|10=000|".as_bytes();
         assert!(matches!(decoder.decode(msg), Err(DecodeError::CheckSum)));
@@ -364,29 +361,22 @@ mod test {
     #[test]
     fn edge_cases_dont_cause_panic() {
         let decoder = new_decoder();
-        assert!(decoder.decode("8=|9=0|10=225|".as_bytes()).is_err());
-        assert!(decoder.decode("8=|9=0|10=|".as_bytes()).is_err());
-        assert!(decoder.decode("8====|9=0|10=|".as_bytes()).is_err());
-        assert!(decoder.decode("|||9=0|10=|".as_bytes()).is_err());
-        assert!(decoder.decode("9999999999999".as_bytes()).is_err());
-        assert!(decoder.decode("-9999999999999".as_bytes()).is_err());
-        assert!(decoder.decode("==============".as_bytes()).is_err());
-        assert!(decoder.decode("9999999999999|".as_bytes()).is_err());
-        assert!(decoder.decode("|999999999999=|".as_bytes()).is_err());
-        assert!(decoder
-            .decode("|999=999999999999999999|=".as_bytes())
-            .is_err());
+        decoder.decode("8=|9=0|10=225|".as_bytes()).ok();
+        decoder.decode("8=|9=0|10=|".as_bytes()).ok();
+        decoder.decode("8====|9=0|10=|".as_bytes()).ok();
+        decoder.decode("|||9=0|10=|".as_bytes()).ok();
+        decoder.decode("9999999999999".as_bytes()).ok();
+        decoder.decode("-9999999999999".as_bytes()).ok();
+        decoder.decode("==============".as_bytes()).ok();
+        decoder.decode("9999999999999|".as_bytes()).ok();
+        decoder.decode("|999999999999=|".as_bytes()).ok();
+        decoder.decode("|999=999999999999999999|=".as_bytes()).ok();
     }
 
-    fn new_decoder_buffered() -> RawDecoderBuffered {
-        let mut config = Config::default();
-        config.set_separator(b'|');
-        RawDecoder::with_config(config).buffered()
-    }
     #[test]
     fn new_buffered_decoder_has_no_current_frame() {
-        let decoder = new_decoder_buffered();
-        assert!(decoder.current_frame().unwrap().is_none());
+        let decoder = new_decoder().buffered();
+        assert!(decoder.raw_frame().unwrap().is_none());
     }
 
     #[test]
@@ -401,13 +391,14 @@ mod test {
             stream
         };
         let mut i = 0;
-        let decoder = &mut new_decoder_buffered();
+        let mut decoder = new_decoder().buffered();
         let mut frame = None;
         while frame.is_none() || i >= stream.len() {
             let buf = decoder.supply_buffer();
             buf.clone_from_slice(&stream[i..i + buf.len()]);
-            i = buf.len();
-            frame = decoder.current_frame().unwrap();
+            i += buf.len();
+            decoder.parse();
+            frame = decoder.raw_frame().unwrap();
         }
         assert!(frame.is_some());
     }
